@@ -11,7 +11,8 @@
 #############################################################################
 
 package XML::Smart::Tree ;
-use Carp ;
+
+no warnings ;
 
 our ($VERSION) ;
 $VERSION = '1.0' ;
@@ -19,6 +20,7 @@ $VERSION = '1.0' ;
   my %PARSERS = (
   XML_Parser => 0 ,
   XML_Smart_Parser => 0 ,
+  XML_Smart_HTMLParser => 0 ,
   ) ;
   
   my $LOADED ;
@@ -45,7 +47,17 @@ sub load_XML_Parser {
 
 sub load_XML_Smart_Parser {
   eval('use XML::Smart::Parser ;') ;
-  if ($@) { print "/// $@\n" ; return( undef ) ;}
+  if ($@) { return( undef ) ;}
+  return(1) ;
+}
+
+#############################
+# LOAD_XML_SMART_HTMLPARSER #
+#############################
+
+sub load_XML_Smart_HTMLParser {
+  eval('use XML::Smart::HTMLParser ;') ;
+  if ($@) { return( undef ) ;}
   return(1) ;
 }
 
@@ -60,6 +72,10 @@ sub load {
   if ($parser) {
     $parser =~ s/:+/_/gs ;
     $parser =~ s/\W//g ;
+    
+    if    ($parser =~ /^html?$/i) { $parser = 'XML_Smart_HTMLParser' ;}
+    elsif ($parser =~ /^(?:re|smart)/i) { $parser = 'XML_Smart_Parser' ;}
+    
     foreach my $Key ( keys %PARSERS ) {
       if ($Key =~ /^$parser$/i) { $module = $Key ; last ;}
     }
@@ -71,6 +87,9 @@ sub load {
   elsif ($module eq 'XML_Smart_Parser') {
     $PARSERS{XML_Smart_Parser} = 1 if &load_XML_Smart_Parser() ;
   }
+  elsif ($module eq 'XML_Smart_HTMLParser') {
+    $PARSERS{XML_Smart_HTMLParser} = 1 if &load_XML_Smart_HTMLParser() ;
+  }
   elsif (!$LOADED) {
     $PARSERS{XML_Parser} = 1 if &load_XML_Parser() ;
     $module = 'XML_Parser' ;
@@ -89,7 +108,6 @@ sub load {
 #########
 
 sub parse {
-  my $xml ;
   my $module = $_[1] ;
   
   my $data ;
@@ -97,6 +115,7 @@ sub parse {
     my ($fh,$open) ;
     
     if (ref($_[0]) eq 'GLOB') { $fh = $_[0] ;}
+    elsif ($_[0] =~ /^http:\/\/\w+[^\r\n]+$/s) { $data = &get_url($_[0]) ;}
     elsif ($_[0] =~ /<.*?>/s) { $data = $_[0] ;}
     else { open ($fh,$_[0]) ; binmode($fh) ; $open = 1 ;}
     
@@ -107,17 +126,28 @@ sub parse {
   }
   
   if ($data !~ /<.*?>/s) { return( {} ) ;}
-
   
   if (!$module || !$PARSERS{$module}) {
     if    ($PARSERS{XML_Parser}) { $module = 'XML_Parser' ;}
     elsif ($PARSERS{XML_Smart_Parser}) { $module = 'XML_Smart_Parser' ;}
   }
   
+  my $xml ;
   if ($module eq 'XML_Parser') { $xml = XML::Parser->new() ;}
   elsif ($module eq 'XML_Smart_Parser') { $xml = XML::Smart::Parser->new() ;}
+  elsif ($module eq 'XML_Smart_HTMLParser') { $xml = XML::Smart::HTMLParser->new() ;}
   else { croak("Can't find a parser for XML!") ;}
-    
+  
+  shift(@_) ;
+  if ( $_[0] =~ /^\s*(?:XML_\w+|html?|re\w+|smart)\s*$/i) { shift(@_) ;}
+
+  my ( %args ) = @_ ;
+  
+  if ( $args{lowtag} ) { $xml->{SMART}{tag} = 1 ;}
+  if ( $args{upertag} ) { $xml->{SMART}{tag} = 2 ;}
+  if ( $args{lowarg} ) { $xml->{SMART}{arg} = 1 ;}
+  if ( $args{uperarg} ) { $xml->{SMART}{arg} = 2 ;}
+  
   $xml->setHandlers(
   Init => \&_Init ,
   Start => \&_Start ,
@@ -130,6 +160,30 @@ sub parse {
   return( $tree ) ;
 }
 
+###########
+# GET_URL #
+###########
+
+sub get_url {
+  my ( $url ) = @_ ;
+  my $data ;
+  
+  require LWP ;
+  require LWP::UserAgent ;
+
+  my $ua = LWP::UserAgent->new();
+  
+  my $agent = $ua->agent() ;
+  $agent = "XML::Smart/$XML::Smart::VERSION $agent" ;
+  $ua->agent($agent) ;
+
+  my $req = HTTP::Request->new(GET => $url) ;
+  my $res = $ua->request($req) ;
+
+  if ($res->is_success) { return $res->content ;}
+  else { return undef ;}
+}
+
 ##########
 # MODULE #
 ##########
@@ -159,12 +213,42 @@ sub _Init {
 # _START #
 ##########
 
-sub _Start {
+sub _Start { #print "START>> @_[1]\n" ;
   my $this = shift ;
   my ($tag , %args) = @_ ;
+  
+  if    ( $this->{SMART}{tag} == 1 ) { $tag = lc($tag) ;}
+  elsif ( $this->{SMART}{tag} == 2 ) { $tag = uc($tag) ;}
+  
+  if ( $this->{SMART}{arg} ) {
+    my $type = $this->{SMART}{arg} ;
+    my %argsok ;
+    foreach my $Key ( keys %args ) {
+      my $k ;
+      if    ($type == 1) { $k = lc($Key) ;}
+      elsif ($type == 2) { $k = uc($Key) ;}
+      
+      if (exists $argsok{$k}) {
+        if ( ref $argsok{$k} ne 'ARRAY' ) {
+          my $key = $argsok{$k} ; 
+          $argsok{$k} = [$key] ;
+        }
+        push(@{$argsok{$k}} , $args{$Key}) ;
+      }
+      else { $argsok{$k} = $args{$Key} ;}
+    }
+    
+    %args = %argsok ;
+  }
 
   $args{'/tag'} = $tag ;
   $args{'/back'} = $this->{PARSING}{p} ;
+
+  push( @{$this->{PARSING}{p}{'/keys'}} , $tag) ;
+  
+  if ($this->{NOENTITY}) {
+    foreach my $Key ( keys %args ) { &XML::Smart::_parse_basic_entity( $args{$Key} ) ;}
+  }
   
   if ( defined $this->{PARSING}{p}{$tag} ) {
     if ( ref($this->{PARSING}{p}{$tag}) ne 'ARRAY' ) {
@@ -190,8 +274,22 @@ sub _Start {
 
 sub _Char {
   my $this = shift ;
-  #print "CONT>> @_\n" ;
-  $this->{PARSING}{p}{CONTENT} .= $_[0] ;
+  #print "CONT>> ##@_##\n" ;
+
+  my $content = $_[0] ;
+  
+  if (! defined $this->{PARSING}{p}{'dt:dt'} && defined $this->{PARSING}{p}{'DT:DT'}) {
+    $this->{PARSING}{p}{'dt:dt'} = delete $this->{PARSING}{p}{'DT:DT'} ;
+  }
+  
+  if ( $this->{PARSING}{p}{'dt:dt'} =~ /binary\.base64/si ) {
+    require XML::Smart::Base64 ;
+    $content = &XML::Smart::Base64::decode_base64($content) ;
+    delete $this->{PARSING}{p}{'dt:dt'} ;
+  }
+  elsif ($this->{NOENTITY}) { &XML::Smart::_parse_basic_entity($content) ;}
+  
+  $this->{PARSING}{p}{CONTENT} .= $content ;
   push(@{$this->{content_ref}} , $this->{PARSING}{p} ) ;
 }
 
@@ -199,12 +297,18 @@ sub _Char {
 # _END #
 ########
 
-sub _End {
+sub _End { #print "END>> @_[1]\n" ;
   my $this = shift ;
-  if ( $this->{PARSING}{p}{'/tag'} ne $_[0] ) { return ;}
+  my $tag = shift ;
+  
+  if    ( $this->{SMART}{tag} == 1 ) { $tag = lc($tag) ;}
+  elsif ( $this->{SMART}{tag} == 2 ) { $tag = uc($tag) ;}
 
+  if ( $this->{PARSING}{p}{'/tag'} ne $tag ) { return ;}
+
+  delete $this->{PARSING}{p}{'/tag'} ;
+  
   my $back  = delete $this->{PARSING}{p}{'/back'} ;
-  my $tag = delete $this->{PARSING}{p}{'/tag'} ;
   my $i = delete $this->{PARSING}{p}{'/i'} || 0 ;
   
   my $nkeys = keys %{$this->{PARSING}{p}} ;
